/*
 * @package netNotes
 * @author Sebastian Tschan
 * @copyright (c) Sebastian Tschan
 * @license GNU General Public License
 * @link https://blueimp.net/mozilla/
 */

let netNotes = {
	prefs: Application.extensions.get('netNotes@blueimp.net').prefs,
	
	initialize: function() {
		// Remove the focus from the keyboard shortcut field on document load:
		document.getElementById('netNotesOptions').focus();
		var folderPref = netNotes.prefs.get('bookmarksFolderID');
		// Ensure the bookmarksFolderID points to a valid bookmarks folder ID:
		if(!netNotes.isBookmarkFolder(folderPref.value)) {
			folderPref.value = PlacesUtils.unfiledBookmarksFolderId;
		}
		gEditItemOverlay.initPanel(folderPref);
	},

	finalize: function() {
		gEditItemOverlay.uninitPanel();
	},
	
	isBookmarkFolder: function() {
		try{
			if(PlacesUtils.bookmarks.getItemType(itemID) == PlacesUtils.bookmarks.TYPE_FOLDER) {
				return true;
			}
		}catch(e){
			// Catch NS_ERROR_ILLEGAL_VALUE
		}
		return false;
	},

	getKeyCodes: function() {
		var keycodes = new Array();
		// Get the list of keycodes from the global KeyEvent object:
		for(var property in KeyEvent) {
			keycodes[KeyEvent[property]] = property.replace('DOM_','');
		}
		// VK_BACK_SPACE (index 8) must be VK_BACK:
		keycodes[8] = 'VK_BACK';
		return keycodes;
	},
	
	formatKeyboardShortcut: function(keyboardShortcut) {
		var stringBundle = document.getElementById('netNotesStringBundle');
		var formattedShortcut = '';
		var items = keyboardShortcut.split('+');
		if(items[0]) {
			// Remove the last element and save it as key:
			var key = items.pop();
			// The remaining items are the modifiers:
			items.forEach(
				function(modifier) {
					try{
						formattedShortcut += stringBundle.getString(modifier) + '+';
					}catch(e){
						Application.console.log(e);
						// Error in shortcut string, return empty string;
						return '';
					}
				}
			);
			// Check if the key is a keycode:
			if(key.indexOf('VK') == 0) {
				try{
					formattedShortcut += stringBundle.getString(key);
				}catch(e){
					// No localization available, just use the plain keycode:
					formattedShortcut += key.replace('VK_', '');
				}
			} else {
				if(key == ' ') {
					formattedShortcut += stringBundle.getString('VK_SPACE');
				} else {
					formattedShortcut += key.toUpperCase();
				}
			}
		}
		return formattedShortcut;
	},
	
	setKeyboardShortcut: function(event) {
		var keyboardShortcut = '';
		// Get the modifiers:
		if(event.altKey) {
			keyboardShortcut += 'alt+';
		}
		if(event.ctrlKey) {
			keyboardShortcut += 'control+';
		}
		if(event.metaKey) {
			keyboardShortcut += 'meta+';
		}
		if(event.shiftKey) {
			keyboardShortcut += 'shift+';
		}
		// Get the key or keycode:
		if(event.keyCode) {
			// Get the keycode from the keycodes list:
			var keycode = netNotes.getKeyCodes()[event.keyCode];
			// Keyboard Shortcut must not be 'VK_TAB' alone (without modifiers),
			// as this button is used to change focus to the next dialog control: 
			if(!keycode || (!keyboardShortcut && keycode == 'VK_TAB')) {
				return;
			}
			keyboardShortcut += keycode;
		} else {
			keyboardShortcut += String.fromCharCode(event.charCode);
		}
		netNotes.prefs.get('keyboardShortcut').value = keyboardShortcut;
	},
	
	disableKeyboardShortcut: function() {
		netNotes.prefs.get('keyboardShortcut').value = '';
	},
	
	openHelp: function(topic) {
		var helpURI = netNotes.prefs.get('helpURI').value.replace(/\[TOPIC\]$/, topic);
		Application.activeWindow.open(this.Utils.createURI(helpURI,null)).focus();
		this.Utils.getBrowser().contentWindow.focus();
	}
}

Components.utils.import('resource://netNotes/utils.js',netNotes);

// Override some methods of the included bookmark folder selection panel:
{
	gEditItemOverlay.initPanel = function(folderPref) {
		this._folderPref = folderPref;
		this._folderMenuList = this._element('folderMenuList');
		this._folderTree = this._element('folderTree');
		this._initFolderMenuList(this._folderPref.value);
		PlacesUtils.bookmarks.addObserver(this, false);
	}
	
	gEditItemOverlay.uninitPanel = function() {
		PlacesUtils.bookmarks.removeObserver(this);
	}
	
	gEditItemOverlay.onFolderMenuListCommand = function(aEvent) {
		if(aEvent.target.id == 'editBMPanel_chooseFolderMenuItem') {
			// reset the selection back to where it was and expand the tree
			// (this menu-item is hidden when the tree is already visible
			var item = this._getFolderMenuItem(this._folderPref.value);
			this._folderMenuList.selectedItem = item;
			// XXXmano HACK: setTimeout 100, otherwise focus goes back to the
			// menulist right away
			setTimeout(
				function(self) self.toggleFolderTreeDisplay(),
				100,
				this
			);
			return;
		}
		// Set the new folderID:
		var container = this._getFolderIdFromMenuList();
		this._folderPref.value = container;	
		// Mark the containing folder as recently-used if it isn't in the static list:
		if(container != PlacesUtils.unfiledBookmarksFolderId &&
			container != PlacesUtils.toolbarFolderId &&
			container != PlacesUtils.bookmarksMenuFolderId) {
			this._markFolderAsRecentlyUsed(container);
		}
		// Update folder-tree selection
		if (!this._folderTree.collapsed) {
			var selectedNode = this._folderTree.selectedNode;
			if(!selectedNode || PlacesUtils.getConcreteItemId(selectedNode) != container) {
	        	this._folderTree.selectItems([container]);
			}
		}	
	}
	
	gEditItemOverlay.toggleFolderTreeDisplay = function(aEvent) {
		this.toggleFolderTreeVisibility();
		window.sizeToContent();
	}
}

window.addEventListener('load', netNotes.initialize, false);
window.addEventListener('unload', netNotes.finalize, false);